export default {
  GROUP: {
    nameKey: "free_busy.types.groups",
    id: "GROUP",
    icon: "mdi-account-group-outline",
    title: "shortName",
  },
  PERSON: {
    nameKey: "free_busy.types.persons",
    id: "PERSON",
    icon: "mdi-account-outline",
    title: "fullName",
  },
};
