query celeryProgressByUser {
  celeryProgressByUser {
    state
    success
    progress {
      current
      total
      percent
    }
    complete
    meta {
      id
      title
      progressTitle
    }
  }
}
