mutation changePassword($oldPassword: String, $userId: ID, $password: String!) {
  changePassword: changeUserPassword(
    password: $password
    oldPassword: $oldPassword
    userId: $userId
  ) {
    success
  }
}

query personUserPw($id: ID) {
  personUserPw: personByIdOrMe(id: $id) {
    id
    userid
    fullName
    canChangePassword
  }
}
