# Generated by Django 4.2.8 on 2023-12-17 18:54

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("core", "0083_availabilityevent"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="group",
            options={
                "ordering": ["short_name", "name"],
                "permissions": (
                    (
                        "assign_child_groups_to_groups",
                        "Can assign child groups to groups",
                    ),
                    ("view_group_stats", "Can view statistics about group."),
                    (
                        "view_group_free_busy_feed",
                        "Can view groups aggregated availability feed",
                    ),
                ),
                "verbose_name": "Group",
                "verbose_name_plural": "Groups",
            },
        ),
        migrations.AlterModelOptions(
            name="person",
            options={
                "ordering": ["last_name", "first_name"],
                "permissions": (
                    ("view_address", "Can view address"),
                    ("view_contact_details", "Can view contact details"),
                    ("view_photo", "Can view photo"),
                    ("view_avatar", "Can view avatar image"),
                    ("view_person_groups", "Can view persons groups"),
                    ("view_personal_details", "Can view personal details"),
                    (
                        "view_person_free_busy_feed",
                        "Can view persons aggregated availability feed",
                    ),
                ),
                "verbose_name": "Person",
                "verbose_name_plural": "Persons",
            },
        ),
    ]
