export const algorithmChoices = [
  {
    title: "o_auth_application.algorithm_no_oidc",
    value: "A_",
  },
  {
    title: "o_auth_application.algorithm_rsa",
    value: "RS256",
  },
  {
    title: "o_auth_application.algorithm_hmac",
    value: "HS256",
  },
];

export const clientTypeChoices = [
  {
    title: "o_auth_application.client_type_confidential",
    value: "CONFIDENTIAL",
  },
  {
    title: "o_auth_application.client_type_public",
    value: "PUBLIC",
  },
];
