# Generated by Django 4.2.11 on 2024-05-14 14:18

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0062_oauth_update"),
    ]

    operations = [
        migrations.AddField(
            model_name="announcement",
            name="priority",
            field=models.PositiveSmallIntegerField(blank=True, null=True, verbose_name="Priority"),
        ),
    ]
