# Generated by Django 5.2.4 on 2025-07-14 20:38

import aleksis.core.mixins
import ckeditor.fields
import colorfield.fields
import django.contrib.postgres.fields
import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0092_add_todo'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AddField(
            model_name='announcement',
            name='is_global',
            field=models.BooleanField(default=False, help_text='Show this announcement for everyone', verbose_name='Global announcement'),
        ),
        migrations.CreateModel(
            name='AnnouncementAlarm',
            fields=[
                ('calendaralarm_ptr',
                 models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True,
                                      primary_key=True, serialize=False, to='core.calendaralarm')),
            ],
            options={
                'verbose_name': 'Announcement alarm',
                'verbose_name_plural': 'Announcement alarms',
            },
            bases=('core.calendaralarm',),
        ),
    ]
