# Generated by Django 5.1.3 on 2025-05-09 12:25

import aleksis.core.mixins
import ckeditor.fields
import django.contrib.postgres.fields
import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0086_calendarwidget'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AddField(
            model_name='activity',
            name='uuid',
            field=models.UUIDField(db_default=models.Func(function="gen_random_uuid"), unique=True, editable=False),
        ),
        migrations.AddField(
            model_name='address',
            name='uuid',
            field=models.UUIDField(db_default=models.Func(function="gen_random_uuid"), unique=True, editable=False),
        ),
        migrations.AddField(
            model_name='addresstype',
            name='uuid',
            field=models.UUIDField(db_default=models.Func(function="gen_random_uuid"), unique=True, editable=False),
        ),
        migrations.AddField(
            model_name='announcementrecipient',
            name='uuid',
            field=models.UUIDField(db_default=models.Func(function="gen_random_uuid"), unique=True, editable=False),
        ),
        migrations.AddField(
            model_name='calendaralarm',
            name='uuid',
            field=models.UUIDField(db_default=models.Func(function="gen_random_uuid"), unique=True, editable=False),
        ),
        migrations.AddField(
            model_name='calendarevent',
            name='uuid',
            field=models.UUIDField(db_default=models.Func(function="gen_random_uuid"), unique=True, editable=False),
        ),
        migrations.AddField(
            model_name='custommenu',
            name='uuid',
            field=models.UUIDField(db_default=models.Func(function="gen_random_uuid"), unique=True, editable=False),
        ),
        migrations.AddField(
            model_name='custommenuitem',
            name='uuid',
            field=models.UUIDField(db_default=models.Func(function="gen_random_uuid"), unique=True, editable=False),
        ),
        migrations.AddField(
            model_name='dashboardwidgetconfiguration',
            name='uuid',
            field=models.UUIDField(db_default=models.Func(function="gen_random_uuid"), unique=True, editable=False),
        ),
        migrations.AddField(
            model_name='dashboardwidgetinstance',
            name='uuid',
            field=models.UUIDField(db_default=models.Func(function="gen_random_uuid"), unique=True, editable=False),
        ),
        migrations.AddField(
            model_name='datacheckresult',
            name='uuid',
            field=models.UUIDField(db_default=models.Func(function="gen_random_uuid"), unique=True, editable=False),
        ),
        migrations.AddField(
            model_name='group',
            name='uuid',
            field=models.UUIDField(db_default=models.Func(function="gen_random_uuid"), unique=True, editable=False),
        ),
        migrations.AddField(
            model_name='grouptype',
            name='uuid',
            field=models.UUIDField(db_default=models.Func(function="gen_random_uuid"), unique=True, editable=False),
        ),
        migrations.AddField(
            model_name='notification',
            name='uuid',
            field=models.UUIDField(db_default=models.Func(function="gen_random_uuid"), unique=True, editable=False),
        ),
        migrations.AddField(
            model_name='organisation',
            name='uuid',
            field=models.UUIDField(db_default=models.Func(function="gen_random_uuid"), unique=True, editable=False),
        ),
        migrations.AddField(
            model_name='organisationaddressthrough',
            name='uuid',
            field=models.UUIDField(db_default=models.Func(function="gen_random_uuid"), unique=True, editable=False),
        ),
        migrations.AddField(
            model_name='pdffile',
            name='uuid',
            field=models.UUIDField(db_default=models.Func(function="gen_random_uuid"), unique=True, editable=False),
        ),
        migrations.AddField(
            model_name='person',
            name='uuid',
            field=models.UUIDField(db_default=models.Func(function="gen_random_uuid"), unique=True, editable=False),
        ),
        migrations.AddField(
            model_name='personaddressthrough',
            name='uuid',
            field=models.UUIDField(db_default=models.Func(function="gen_random_uuid"), unique=True, editable=False),
        ),
        migrations.AddField(
            model_name='persongroupthrough',
            name='uuid',
            field=models.UUIDField(db_default=models.Func(function="gen_random_uuid"), unique=True, editable=False),
        ),
        migrations.AddField(
            model_name='personrelationship',
            name='uuid',
            field=models.UUIDField(db_default=models.Func(function="gen_random_uuid"), unique=True, editable=False),
        ),
        migrations.AddField(
            model_name='role',
            name='uuid',
            field=models.UUIDField(db_default=models.Func(function="gen_random_uuid"), unique=True, editable=False),
        ),
        migrations.AddField(
            model_name='room',
            name='uuid',
            field=models.UUIDField(db_default=models.Func(function="gen_random_uuid"), unique=True, editable=False),
        ),
        migrations.AddField(
            model_name='schoolterm',
            name='uuid',
            field=models.UUIDField(db_default=models.Func(function="gen_random_uuid"), unique=True, editable=False),
        ),
        migrations.AddField(
            model_name='taskuserassignment',
            name='uuid',
            field=models.UUIDField(db_default=models.Func(function="gen_random_uuid"), unique=True, editable=False),
        ),
    ]
