# Generated by Django 4.2.5 on 2023-10-13 12:07

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0082_announcement_remove_fields"),
    ]

    operations = [
        migrations.CreateModel(
            name="AvailabilityEvent",
            fields=[
                (
                    "calendarevent_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="core.calendarevent",
                    ),
                ),
                ("title", models.CharField(blank=True, max_length=255, verbose_name="Title")),
                ("description", models.TextField(blank=True, verbose_name="Description")),
                ("free", models.BooleanField(verbose_name="Free")),
                (
                    "person",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="availability_events",
                        to="core.person",
                        verbose_name="Person",
                    ),
                ),
            ],
            options={
                "verbose_name": "Availability event",
                "verbose_name_plural": "Availability events",
            },
            bases=("core.calendarevent",),
        ),
    ]
